@@ -20,13 +20,12 @@
 """Manages the magnifier for orca.  [[[TODO: WDW - this is very very
 early in development.  One might even say it is pre-prototype.]]]"""
 
-__id__        = "$Id: mag.py 3882 2008-05-07 18:22:10Z richb $"
-__version__   = "$Revision: 3882 $"
-__date__      = "$Date: 2008-05-07 14:22:10 -0400 (Wed, 07 May 2008) $"
+__id__        = "$Id$"
+__version__   = "$Revision$"
+__date__      = "$Date$"
 __copyright__ = "Copyright (c) 2005-2008 Sun Microsystems Inc."
 __license__   = "LGPL"
 
-import bonobo
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -50,6 +49,7 @@ from orca_i18n import _  # for gettext support
 _magnifierAvailable = False
 
 try:
+    import bonobo
     import ORBit
     ORBit.load_typelib('GNOME_Magnifier')
     import GNOME.Magnifier
@@ -465,6 +465,20 @@ def __setupMagnifier(position, left=None, top=None, right=None, bottom=None,
     except:
         debug.printException(debug.LEVEL_WARNING)
 
+    # If we are running in full screen mode, try to hide the original cursor
+    # (assuming the user wants to). See bug #533095 for more details.
+    # Depends upon new functionality in gnome-mag, so just catch the 
+    # exception if this functionality isn't there.
+    #
+    hideCursor = restore.get('magHideCursor', settings.magHideCursor)
+    if hideCursor \
+       and _fullScreenCapable \
+       and _magnifier.SourceDisplay == _magnifier.TargetDisplay \
+       and position == settings.MAG_ZOOMER_TYPE_FULL_SCREEN:
+        hideSystemPointer(True)
+    else:
+        hideSystemPointer(False)
+
     _magnifierPBag = _magnifier.getProperties()
     sdb = _magnifierPBag.getValue("source-display-bounds").value()
     if not _originalSourceDisplayBounds:
@@ -989,6 +1003,7 @@ def shutdown():
     # See http://bugzilla.gnome.org/show_bug.cgi?id=375396.
     #
     try:
+        hideSystemPointer(False)
         _magnifier.clearAllZoomRegions()
         _magnifier.dispose()
     except:
@@ -1377,6 +1392,36 @@ def setZoomerColorFilter(colorFilter, updateScreen=True):
     if updateScreen:
         _zoomer.markDirty(_roi)
 
+def hideSystemPointer(hidePointer):
+    """Hide or show the system pointer.
+
+    Arguments:
+    -hidePointer: If True, hide the system pointer, otherwise show it.
+    """
+
+    # Depends upon new functionality in gnome-mag, so just catch the
+    # exception if this functionality isn't there.
+    #
+    try:
+        if hidePointer:
+            _magnifier.hideCursor()
+        else:
+            _magnifier.showCursor()
+    except:
+        debug.printException(debug.LEVEL_FINEST)
+
+def isFullScreenCapable():
+    """Returns True if we are capable of doing full screen (i.e. whether
+    composite is being used.
+    """
+
+    try:
+        capable = _magnifier.fullScreenCapable()
+    except:
+        capable = False
+
+    return capable
+
 def isFilteringCapable():
     """Returns True if we're able to take advantage of libcolorblind's color
     filtering.
