@@ -27,7 +27,6 @@ __license__   = "LGPL"
 
 import pyatspi
 
-import orca.braille as braille
 import orca.debug as debug
 import orca.default as default
 import orca.orca as orca
@@ -99,7 +98,8 @@ class Script(default.Script):
             #print "COMPLETED", context.utterance, context.currentOffset
             obj = context.obj[len(context.obj)-1]
             obj.queryText().setCaretOffset(context.currentOffset)
-            orca.setLocusOfFocus(None, obj, False)
+            orca.setLocusOfFocus(
+                None, obj, notifyPresentationManager=False)
 
     def textLines(self, obj):
         """Creates a generator that can be used to iterate over each line
@@ -147,8 +147,8 @@ class Script(default.Script):
 
                 if mode == pyatspi.TEXT_BOUNDARY_LINE_START or \
                    len(mystr) == 0 or mystr[len(mystr)-1] in '.?!':
-                    string = self.adjustForRepeats(string)
-                    if string.isupper():
+                    string = self.utilities.adjustForRepeats(string)
+                    if string.decode("UTF-8").isupper():
                         voice = settings.voices[settings.UPPERCASE_VOICE]
                     else:
                         voice = settings.voices[settings.DEFAULT_VOICE]
@@ -185,8 +185,8 @@ class Script(default.Script):
         # If there is anything left unspoken, speak it now.
         #
         if len(string) != 0:
-            string = self.adjustForRepeats(string)
-            if string.isupper():
+            string = self.utilities.adjustForRepeats(string)
+            if string.decode("UTF-8").isupper():
                 voice = settings.voices[settings.UPPERCASE_VOICE]
             else:
                 voice = settings.voices[settings.DEFAULT_VOICE]
@@ -218,7 +218,8 @@ class Script(default.Script):
         # Spelling dialog. Look for the one that isn't a label to
         # another component.
         #
-        allLabels = self.findByRole(panel, pyatspi.ROLE_LABEL)
+        allLabels = self.utilities.descendantsWithRole(
+            panel, pyatspi.ROLE_LABEL)
         for label in allLabels:
             # Translators: these are labels from the gedit spell checking
             # dialog and must be the same strings gedit uses.  We hate
@@ -239,7 +240,8 @@ class Script(default.Script):
         # was called. If they are the same then we ignore it.
 
         if self.textArea != None:
-            allText = self.findByRole(self.textArea, pyatspi.ROLE_TEXT)
+            allText = self.utilities.descendantsWithRole(
+                self.textArea, pyatspi.ROLE_TEXT)
             caretPosition = allText[0].queryText().caretOffset
 
             debug.println(self.debugLevel, \
@@ -272,7 +274,7 @@ class Script(default.Script):
             #
             allTokens = []
             for i in range(0, len(allText)):
-                text = self.getText(allText[i], 0, -1)
+                text = self.utilities.substring(allText[i], 0, -1)
                 tokens = text.split()
                 allTokens += tokens
 
@@ -314,9 +316,9 @@ class Script(default.Script):
         # we're forced to do so in this case.
         #
         tmp = obj.parent.parent
-        if (self.isDesiredFocusedItem(obj, rolesList1) \
+        if (self.utilities.hasMatchingHierarchy(obj, rolesList1) \
             and obj.name == _("Find")) \
-            or (self.isDesiredFocusedItem(obj, rolesList2) \
+            or (self.utilities.hasMatchingHierarchy(obj, rolesList2) \
                 and tmp.parent.parent.parent.name == _("Find")):
             return True
         else:
@@ -357,7 +359,7 @@ class Script(default.Script):
                      pyatspi.ROLE_PAGE_TAB,
                      pyatspi.ROLE_PAGE_TAB_LIST,
                      pyatspi.ROLE_SPLIT_PANE]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if self.utilities.hasMatchingHierarchy(event.source, rolesList):
             debug.println(self.debugLevel,
                           "gedit.locusOfFocusChanged - text area.")
 
@@ -381,7 +383,7 @@ class Script(default.Script):
                      pyatspi.ROLE_PANEL,
                      pyatspi.ROLE_FILLER,
                      pyatspi.ROLE_FRAME]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if self.utilities.hasMatchingHierarchy(event.source, rolesList):
             tmp = event.source.parent.parent
             frame = tmp.parent.parent
             # Translators: this is the name of the "Check Spelling" window
@@ -414,20 +416,15 @@ class Script(default.Script):
                      pyatspi.ROLE_FILLER,
                      pyatspi.ROLE_PAGE_TAB,
                      pyatspi.ROLE_PAGE_TAB_LIST]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if self.utilities.hasMatchingHierarchy(event.source, rolesList):
             debug.println(self.debugLevel,
                           "gedit.onStateChanged - print preview - page #.")
-            line = braille.getShowingLine()
             parent = event.source.parent
-            label1 = self.getDisplayedText(parent[1])
-            label2 = self.getDisplayedText(parent[2])
-
-            utterances = [ label1, label2 ]
-            line.addRegion(braille.Region(" " + label1))
-            line.addRegion(braille.Region(" " + label2))
-
-            speech.speakUtterances(utterances)
-            braille.refresh()
+            label1 = self.utilities.displayedText(parent[1])
+            label2 = self.utilities.displayedText(parent[2])
+            items = [label1, label2]
+            self.presentItemsInSpeech(items)
+            self.presentItemsInBraille(items)
 
     # This method tries to detect and handle the following cases:
     # 1) check spelling dialog.
@@ -465,7 +462,7 @@ class Script(default.Script):
                      pyatspi.ROLE_PANEL,
                      pyatspi.ROLE_FILLER,
                      pyatspi.ROLE_FRAME]
-        if self.isDesiredFocusedItem(event.source, rolesList):
+        if self.utilities.hasMatchingHierarchy(event.source, rolesList):
             frame = event.source.parent.parent.parent
             # Translators: this is the name of the "Check Spelling" window
             # in gedit and must be the same as what gedit uses.  We hate
@@ -491,6 +488,8 @@ class Script(default.Script):
         # apparently get two identical "object:property-change:accessible-name"
         # events.]]]
 
+        lastKey, mods = self.utilities.lastKeyAndModifiers()
+
         # Translators: the "Phrase not found" is the result of a failed
         # find command.  It must be the same as what gedit uses.  We hate
         # keying off stuff like this, but we're forced to do so in this
@@ -498,11 +497,10 @@ class Script(default.Script):
         #
         if event.source.getRole() == pyatspi.ROLE_STATUS_BAR \
            and self.isFocusOnFindDialog() \
-           and orca_state.lastNonModifierKeyEvent.event_string == "Return" \
+           and lastKey == "Return" \
            and event.source.name == _("Phrase not found"):
             debug.println(self.debugLevel,
                           "gedit.onNameChanged - phrase not found.")
-
             speech.speak(event.source.name)
 
         # Pass the event onto the parent class to be handled in the default way.
@@ -552,13 +550,13 @@ class Script(default.Script):
         # then speak the current text line, to give an indication of what
         # we've just found.
         #
-        if self.isFocusOnFindDialog() \
-           and orca_state.lastNonModifierKeyEvent.event_string == "Return":
+        lastKey, mods = self.utilities.lastKeyAndModifiers()
+        if self.isFocusOnFindDialog() and lastKey == "Return":
             debug.println(self.debugLevel, "gedit.onCaretMoved - find dialog.")
-            allComboBoxes = \
-                     self.findByRole(orca_state.locusOfFocus.getApplication(),
-                                     pyatspi.ROLE_COMBO_BOX)
-            phrase = self.getDisplayedText(allComboBoxes[0])
+            allComboBoxes = self.utilities.descendantsWithRole(
+                orca_state.locusOfFocus.getApplication(),
+                pyatspi.ROLE_COMBO_BOX)
+            phrase = self.utilities.displayedText(allComboBoxes[0])
             [text, caretOffset, startOffset] = \
                 self.getTextLineAtCaret(event.source)
             if text.lower().find(phrase) != -1:
@@ -566,16 +564,14 @@ class Script(default.Script):
                 # finding something.
                 #
                 speech.speak(_("Phrase found."))
-                utterances = self.speechGenerator.getSpeech(event.source, True)
-                speech.speakUtterances(utterances)
+                utterances = self.speechGenerator.generateSpeech(
+                    event.source, alreadyFocused=True)
+                speech.speak(utterances)
 
         # If Ctrl+G was used to repeat a find command, speak the line that
         # the caret moved to.
         #
-        if orca_state.lastInputEvent \
-           and orca_state.lastInputEvent.event_string == 'G' \
-           and orca_state.lastInputEvent.modifiers \
-               & (1 << pyatspi.MODIFIER_CONTROL):
+        if lastKey == 'G' and mods & settings.CTRL_MODIFIER_MASK:
             self.sayLine(event.source)
 
         # For everything else, pass the caret moved event onto the parent
