@@ -116,15 +116,6 @@ class Script(default.Script):
         #
         self.commFailureAttemptLimit = 1
 
-        # Additional default set of text attributes to present to the
-        # user. The only difference over the default set in
-        # settings.py is to add in "left-margin:" and "right-margin:".
-        #
-        self.additionalBrailledTextAttributes = \
-          " left-margin:0mm; right-margin:0mm;"
-        self.additionalSpokenTextAttributes = \
-          " left-margin:0mm; right-margin:0mm;"
-
     def activate(self):
         """Called when this script is activated."""
         self.savedreadTableCellRow = settings.readTableCellRow
@@ -132,17 +123,18 @@ class Script(default.Script):
 
         self.savedEnabledBrailledTextAttributes = \
             settings.enabledBrailledTextAttributes
-        settings.enabledBrailledTextAttributes += \
-            self.additionalBrailledTextAttributes
 
         self.savedEnabledSpokenTextAttributes = \
             settings.enabledSpokenTextAttributes
-        settings.enabledSpokenTextAttributes += \
-            self.additionalSpokenTextAttributes
 
-        # Account for the differences in how OOo expresses indent and
-        # strikethrough.
+        # Account for the differences in how OOo expresses indent, 
+        # strikethrough, and margins.
         #
+        settings.allTextAttributes = \
+            settings.allTextAttributes.replace(
+            "margin:;",
+            "margin:0mm;")
+
         settings.enabledBrailledTextAttributes = \
             settings.enabledBrailledTextAttributes.replace(
             "strikethrough:false;",
@@ -1273,9 +1265,11 @@ class Script(default.Script):
             hypertext = None
 
         if not hypertext or (hypertext.getNLinks() == 0):
-            if settings.enableSpeechIndentation:
-                self.speakTextIndentation(event.source,
-                                          textToSpeak.encode("UTF-8"))
+            result = self.speechGenerator.generateTextIndentation( \
+              event.source, line=textToSpeak.encode("UTF-8"))
+            if result:
+                speech.speak(result[0])
+
             speech.speak(textToSpeak.encode("UTF-8"), None, False)
         else:
             started = False
@@ -1734,6 +1728,13 @@ class Script(default.Script):
                 orca_state.locusOfFocus.parent, event.source):
             return
 
+        # Auto-inserted bullets and numbers are presented in braille, but not
+        # spoken. So we'll speak them before sending this event off to the
+        # default script.
+        #
+        if self.utilities.isAutoTextEvent(event):
+            speech.speak(self.speechGenerator.generateSpeech(event.source))
+
         default.Script.onFocus(self, event)
 
     def onActiveDescendantChanged(self, event):
@@ -1767,8 +1768,13 @@ class Script(default.Script):
             # The style list in the Formatting toolbar also lacks state
             # focused.
             #
-            elif event.any_data and self.utilities.ancestorWithRole(
-                event.source, [pyatspi.ROLE_TOOL_BAR], [pyatspi.ROLE_FRAME]):
+            elif event.any_data \
+                 and self.utilities.ancestorWithRole(event.source,
+                                                     [pyatspi.ROLE_TOOL_BAR],
+                                                     [pyatspi.ROLE_FRAME]) \
+                 and self.utilities.ancestorWithRole(orca_state.locusOfFocus,
+                                                     [pyatspi.ROLE_TOOL_BAR],
+                                                     [pyatspi.ROLE_FRAME]):
                 handleEvent = True
 
         elif self.utilities.isSameObject(
@@ -1876,12 +1882,11 @@ class Script(default.Script):
                 y = orca_state.lastInputEvent.y
                 weToggledIt = event.source.queryComponent().contains(x, y, 0)
 
-            elif isinstance(orca_state.lastInputEvent, \
-                            input_event.KeyboardEvent):
-                keyString = orca_state.lastNonModifierKeyEvent.event_string
-                navKeys = ["Up", "Down", "Page_Up", "Page_Down", "Home", "End"]
-                wasCommand = orca_state.lastInputEvent.modifiers \
-                             & settings.COMMAND_MODIFIER_MASK
+            else:
+                keyString, mods = self.utilities.lastKeyAndModifiers()
+                navKeys = ["Up", "Down", "Left", "Right", "Page_Up",
+                           "Page_Down", "Home", "End"]
+                wasCommand = mods & settings.COMMAND_MODIFIER_MASK
                 weToggledIt = wasCommand and keyString not in navKeys
 
             if weToggledIt:
@@ -2061,16 +2066,13 @@ class Script(default.Script):
             if settings.enableEchoByWord and \
                (self.utilities.hasMatchingHierarchy(event.source, rolesList) or
                 self.utilities.hasMatchingHierarchy(event.source, rolesList1)):
-                if isinstance(orca_state.lastInputEvent,
-                              input_event.KeyboardEvent):
-                    keyString = orca_state.lastNonModifierKeyEvent.event_string
-                    focusRole = orca_state.locusOfFocus.getRole()
-                    if focusRole != pyatspi.ROLE_UNKNOWN and \
-                       keyString == "Return":
-                        result = self.utilities.substring(event.source, 0, -1)
-                        line = result.decode("UTF-8")
-                        self.echoPreviousWord(event.source, len(line))
-                        return
+                keyString, mods = self.utilities.lastKeyAndModifiers()
+                focusRole = orca_state.locusOfFocus.getRole()
+                if focusRole != pyatspi.ROLE_UNKNOWN and keyString == "Return":
+                    result = self.utilities.substring(event.source, 0, -1)
+                    line = result.decode("UTF-8")
+                    self.echoPreviousWord(event.source, len(line))
+                    return
 
         # Otherwise, if the object is losing focus, then just ignore this event.
         #
@@ -2090,13 +2092,7 @@ class Script(default.Script):
                 #
                 self.lastCell[1] = event.detail1
 
-        if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent) \
-           and orca_state.lastNonModifierKeyEvent:
-            event_string = orca_state.lastNonModifierKeyEvent.event_string
-            mods = orca_state.lastInputEvent.modifiers
-        else:
-            event_string = ''
-            mods = 0
+        event_string, mods = self.utilities.lastKeyAndModifiers()
         isControlKey = mods & settings.CTRL_MODIFIER_MASK
         isShiftKey = mods & settings.SHIFT_MODIFIER_MASK
 
@@ -2130,11 +2126,15 @@ class Script(default.Script):
             # object:active-descendant-changed events which cause us
             # to set the locusOfFocus to the list item. If the user then
             # arrows within the text portion, we will not present it due
-            # to the event not being from the locusOfFocus.
+            # to the event not being from the locusOfFocus. A similar
+            # issue is present in the Target entry of the Hyperlink dialog
+            # for OOo 3.2.
             #
             if event.source.getRole() == pyatspi.ROLE_TEXT \
                and self.utilities.ancestorWithRole(
-                event.source, [pyatspi.ROLE_TOOL_BAR], [pyatspi.ROLE_FRAME]):
+                    event.source, 
+                    [pyatspi.ROLE_TOOL_BAR, pyatspi.ROLE_DIALOG],
+                    [pyatspi.ROLE_FRAME]):
                 orca.setLocusOfFocus(event, event.source, False)
             default.Script.onCaretMoved(self, event)
 
