@@ -30,6 +30,7 @@ import pyatspi
 import orca.speech_generator as speech_generator
 import orca.settings as settings
 
+from orca.orca_i18n import ngettext # for ngettext support
 from orca.orca_i18n import _ # for gettext support
 
 import script_settings
@@ -49,9 +50,8 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         override = \
             role == "text frame" \
             or (role == pyatspi.ROLE_PARAGRAPH \
-                and self._script.getAncestor(obj,
-                                             [pyatspi.ROLE_DIALOG],
-                                             [pyatspi.ROLE_APPLICATION]))
+                and self._script.utilities.ancestorWithRole(
+                      obj, [pyatspi.ROLE_DIALOG], [pyatspi.ROLE_APPLICATION]))
         return override
 
     def _generateRoleName(self, obj, **args):
@@ -92,14 +92,14 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
     def _generateLabel(self, obj, **args):
         """Returns the label for an object as an array of strings (and
         possibly voice and audio specifications).  The label is
-        determined by the getDisplayedLabel of the script, and an
-        empty array will be returned if no label can be found.
+        determined by the displayedLabel method of the script utility,
+        and an empty array will be returned if no label can be found.
         """
         result = []
         override = self.__overrideParagraph(obj, **args)
-        label = self._script.getDisplayedLabel(obj) or ""
+        label = self._script.utilities.displayedLabel(obj) or ""
         if not label and override:
-            label = self._script.getDisplayedLabel(obj.parent) or ""
+            label = self._script.utilities.displayedLabel(obj.parent) or ""
         result.append(label.strip())
         return result
 
@@ -142,7 +142,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             # and the displayed text, with punctuation added. Try to spot
             # this and, if found, ignore the description.
             #
-            text = self._script.getDisplayedText(obj) or ""
+            text = self._script.utilities.displayedText(obj) or ""
             desc = obj.description.replace(text, "")
             for item in obj.name.split():
                 desc = desc.replace(item, "")
@@ -187,7 +187,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         except:
             pass
         else:
-            index = self._script.getCellIndex(obj)
+            index = self._script.utilities.cellIndex(obj)
             rowIndex = table.getRowAtIndex(index)
             if rowIndex >= 0 \
                and table in self._script.dynamicRowHeaders:
@@ -199,11 +199,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
         return result
@@ -219,7 +219,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             parentTable = parent.queryTable()
         except NotImplementedError:
             parentTable = None
-        index = self._script.getCellIndex(obj)
+        index = self._script.utilities.cellIndex(obj)
         if "lastRow" in self._script.pointOfReference and \
            self._script.pointOfReference["lastRow"] != \
            parentTable.getRowAtIndex(index):
@@ -232,11 +232,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
         return result
@@ -254,7 +254,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         except:
             pass
         else:
-            index = self._script.getCellIndex(obj)
+            index = self._script.utilities.cellIndex(obj)
             columnIndex = table.getColumnAtIndex(index)
             if columnIndex >= 0 \
                and table in self._script.dynamicColumnHeaders:
@@ -266,11 +266,11 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
         return result
@@ -286,7 +286,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             parentTable = parent.queryTable()
         except NotImplementedError:
             parentTable = None
-        index = self._script.getCellIndex(obj)
+        index = self._script.utilities.cellIndex(obj)
         if "lastColumn" in self._script.pointOfReference and \
            self._script.pointOfReference["lastColumn"] != \
            parentTable.getColumnAtIndex(index):
@@ -299,15 +299,50 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     headerText = None
                 if header.childCount > 0:
                     for child in header:
-                        text = self._script.getText(child, 0, -1)
+                        text = self._script.utilities.substring(child, 0, -1)
                         if text:
                             result.append(text)
                 elif headerText:
-                    text = self._script.getText(header, 0, -1)
+                    text = self._script.utilities.substring(header, 0, -1)
                     if text:
                         result.append(text)
         return result
 
+    def _generateTooLong(self, obj, **args):
+        """If there is text in this spread sheet cell, compare the size of
+        the text within the table cell with the size of the actual table
+        cell and report back to the user if it is larger.
+
+        Returns an indication of how many characters are greater than the size
+        of the spread sheet cell, or None if the message fits.
+        """
+        result = []
+        try:
+            text = obj.queryText()
+            objectText = \
+                self._script.utilities.substring(obj, 0, -1).decode("UTF-8")
+            extents = obj.queryComponent().getExtents(pyatspi.DESKTOP_COORDS)
+        except NotImplementedError:
+            pass
+        else:
+            tooLongCount = 0
+            for i in range(0, len(objectText)):
+                [x, y, width, height] = text.getRangeExtents(i, i + 1, 0)
+                if x < extents.x:
+                    tooLongCount += 1
+                elif (x + width) > extents.x + extents.width:
+                    tooLongCount += len(objectText) - i
+                    break
+            if tooLongCount > 0:
+                # Translators: people can enter a string of text that is
+                # too wide for a spreadsheet cell.  This string will be
+                # spoken if such a cell is encountered.
+                #
+                result = [ngettext("%d character too long",
+                                   "%d characters too long",
+                                   tooLongCount) % tooLongCount]
+        return result
+
     def _generateSpreadSheetCell(self, obj, **args):
         result = []
         if self._script.inputLineForCell == None:
@@ -315,7 +350,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 self._script.locateInputLine(obj)
         try:
             if obj.queryText():
-                objectText = self._script.getText(obj, 0, -1)
+                objectText = self._script.utilities.substring(obj, 0, -1)
                 if (not script_settings.speakSpreadsheetCoordinates \
                     or args.get('formatType', 'unfocused') == 'basicWhereAmI') \
                    and len(objectText) == 0:
@@ -341,6 +376,12 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                     if char.isdigit():
                         result.append(name)
                         break
+
+        tooLong = self._generateTooLong(obj, **args)
+        if tooLong and len(tooLong):
+            result.extend(self._generatePause(obj, **args))
+            result.extend(tooLong)
+
         return result
 
     def _generateRealTableCell(self, obj, **args):
@@ -388,7 +429,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
             if settings.readTableCellRow:
                 parent = obj.parent
                 parentTable = parent.queryTable()
-                index = self._script.getCellIndex(obj)
+                index = self._script.utilities.cellIndex(obj)
                 row = parentTable.getRowAtIndex(index)
                 column = parentTable.getColumnAtIndex(index)
                 # This is an indication of whether we should speak all the
@@ -428,6 +469,21 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
                 result.append(_("blank"))
         return result
 
+    def _generateEndOfTableIndicator(self, obj, **args):
+        """Returns an array of strings (and possibly voice and audio
+        specifications) indicating that this cell is the last cell
+        in the table. Overridden here because Orca keeps saying "end
+        of table" in certain lists (e.g. the Templates and Documents
+        dialog).
+        """
+
+        topLevel = self._script.utilities.topLevelObject(obj)
+        if topLevel and topLevel.getRole() == pyatspi.ROLE_DIALOG:
+            return []
+
+        return (speech_generator.SpeechGenerator._generateEndOfTableIndicator(
+                self, obj, **args))
+
     def generateSpeech(self, obj, **args):
         result = []
         if args.get('formatType', 'unfocused') == 'basicWhereAmI' \
